#
# Copyright (C) 2019 Satoru SATOH <satoru.satoh @ gmail.com>
# License: MIT
#
# pylint: disable=missing-docstring,invalid-name
from __future__ import absolute_import, print_function

import os.path
import unittest
import anyconfig
import configobj

from tests.common import dicts_equal


_CURDIR = os.path.dirname(__file__)
_CID = "configobj"  # .. seealso:: anyconfig_configobj_backend/configobj.py


class Test(unittest.TestCase):

    conf_path = os.path.join(_CURDIR, "res/0.configobj")
    conf_ref = configobj.ConfigObj(conf_path)

    def _co_assert_equals(self, cnf):
        self.assertTrue(dicts_equal(cnf, self.conf_ref.dict()))

    def _load_helper(self, **kwargs):
        try:
            anyconfig.api.load_plugins()
            return anyconfig.load(self.conf_path, **kwargs)
        except anyconfig.UnknownFileTypeError:
            for psr in anyconfig.api.Parsers().list():
                print("%r: type=%r, exts=%r" % (psr, psr.type(),
                                                psr.extensions()))
            raise

    def test_22_load__explicit_use(self):
        cnf = self._load_helper(ac_parser=_CID)
        self._co_assert_equals(cnf)

# vim:sw=4:ts=4:et:



#
# Copyright (C) 2013 - 2018 Satoru SATOH <ssato @ redhat.com>
# License: MIT
#
# pylint: disable=missing-docstring,invalid-name,too-few-public-methods
# pylint: disable=ungrouped-imports
from __future__ import absolute_import
import os.path

from anyconfig.compat import OrderedDict as ODict

import anyconfig_configobj_backend as TT
import tests.common as TBC


_ML_0 = """A multiline value,
that spans more than one line :-)
The line breaks are included in the value."""

CNF_0 = ODict((('keyword1', 'value1'),
               ('keyword 2', 'value 2'),
               ('section 1',
                ODict((('keyword 3', 'value 3'),
                       ('keyword 4', ['value4', 'value 5', 'value 6']),
                       ('sub-section',
                        ODict((('keyword 5', 'value 7'),
                               ('keyword 6', _ML_0),
                               ('sub-sub-section',
                                ODict((('keyword 7', 'value 8'), ))))))))),
               ('section 2',
                ODict((('keyword8', 'value 9'), ('keyword9', 'value10'))))))


class HasParserTrait(TBC.HasParserTrait):

    psr = TT.Parser()
    cnf = CNF_0
    cnf_s = open(os.path.join(TBC.selfdir(), "res/0.configobj")).read()


class Test_10(TBC.Test_10_dumps_and_loads, HasParserTrait):

    load_options = dict(raise_errors=True)
    dump_options = dict(indent_type="  ")


class Test_20(TBC.Test_10_dumps_and_loads, HasParserTrait):

    pass

# vim:sw=4:ts=4:et:



#
# Copyright (C) 2013 - 2019 Satoru SATOH <satoru.satoh @ gmail.com>
# License: MIT
#
r"""Configobj backend:

- Format to support: configobj,
  https://bit.ly/2TgURnL (https://configobj.readthedocs.io)
- Requirements: configobj (https://pypi.python.org/pypi/configobj/)
- Development Status :: 4 - Beta
- Limitations: AFAIK, configobj does not keep the order of configuration items
  and not have options to change this behavior like configparser, so this
  backend does not keep the order of configuration items even if the ac_ordered
  option was used.

- Special options:

  - All options except for 'infile' passed to configobj.ConfigObj.__init__
    should work.

  - See also: http://configobj.readthedocs.io/en/latest/configobj.html

Chnagelog:

.. versionchanged:: 0.5.0

   - Now loading and dumping options are detected automatically from inspection
     result if possible. Also these became not distinguished because these will
     be passed to configobj.Configuration anyway.
"""
from __future__ import absolute_import

import os
import configobj
import anyconfig.backend.base

from anyconfig.compat import getargspec


try:
    _LOAD_OPTS = [a for a in getargspec(configobj.ConfigObj).args
                  if a not in "self infile".split()]
except (TypeError, AttributeError):
    _LOAD_OPTS = ("options configspec encoding interpolation raise_errors"
                  "list_values create_empty file_error stringify"
                  "indent_type default_encoding unrepr write_empty_values"
                  "_inspec").split()


def make_configobj(cnf, **kwargs):
    """
    Make a configobj.ConfigObj initalized with given config 'cnf'.

    :param cnf: Configuration data
    :param kwargs: optional keyword parameters passed to ConfigObj.__init__

    :return: An initialized configobj.ConfigObj instance
    """
    cobj = configobj.ConfigObj(**kwargs)
    cobj.update(cnf)

    return cobj


def load(path_or_strm, container, **opts):
    """
    :param path_or_strm: input config file path or file/file-like object
    :param container: callble to make a container object
    :param opts: keyword options passed to :class:`configobj.ConfigObj`

    :return: Mapping object
    """
    return container(configobj.ConfigObj(path_or_strm, **opts))


class Parser(anyconfig.backend.base.StreamParser,
             anyconfig.backend.base.BinaryFilesMixin):
    """
    Parser for Ini-like config files which configobj supports.
    """
    _cid = "configobj"
    _type = "configobj"
    _priority = 10
    _load_opts = _LOAD_OPTS  # options on dump will be just ignored.
    _dump_opts = _LOAD_OPTS  # Likewise.
    _ordered = True

    load_from_path = load_from_stream = anyconfig.backend.base.to_method(load)

    def dump_to_string(self, cnf, **kwargs):
        """
        Dump config 'cnf' to a string.

        :param cnf: Configuration data to dump
        :param kwargs: backend-specific optional keyword parameters :: dict

        :return: string represents the configuration
        """
        return os.linesep.join(make_configobj(cnf, **kwargs).write())

    def dump_to_stream(self, cnf, stream, **kwargs):
        """
        :param cnf: Configuration data to dump
        :param stream: Config file or file-like object
        :param kwargs: backend-specific optional keyword parameters :: dict
        """
        make_configobj(cnf, **kwargs).write(stream)

# vim:sw=4:ts=4:et:
